@@ -1,115 +1,168 @@
 ;;; Handling of comment boxes.
-;;; Copyright (C) 1991, 1992, 1993, 1994, 1995 Free Software Foundation, Inc.
+;;; Copyright (C) 1991, 92, 93, 94, 95, 96, 97 Free Software Foundation, Inc.
 ;;; Franois Pinard <pinard@iro.umontreal.ca>, April 1991.
-;;;
-;;; I often refill paragraphs inside comments, while stretching or
-;;; shrinking the surrounding box as needed.  This is a real pain to
-;;; do by hand (or with vi! :-).  This GNU Emacs LISP code eases my
-;;; life on this.  It would not be fair giving all sources for a
-;;; package without also giving the means for nicely modifying them.
-;;;
-;;; The function rebox-comment discovers the extent of the boxed
-;;; comments near the cursor, possibly refills the text, then adjusts
-;;; the comment box style.  The function rebox-region does the same,
-;;; except that it takes the current region as a boxed comment.
-;;; Numeric prefixes are used to add or remove a box, change its style
-;;; (language, quality or type), or to prevent refilling of its text.
-;;;
-;;; For most Emacs language editing modes, refilling does not make
-;;; sense outside comments, so you may redefine the M-q command and
-;;; link it to this file.  For example, I use this in my .emacs file:
-;;;
+
+;;; I first observed rounded corners, as in style 223 boxes, in code from
+;;; Warren Tucker <wht@n4hgf.mt-park.ga.us>, a previous shar maintainer.
+
+;;; Refilling paragraphs inside comments, stretching or shrinking the
+;;; surrounding box as needed, is a pain to do "by hand".  This GNU Emacs
+;;; LISP code eases my life on this and I find it fair, giving all sources
+;;; for a package, to also give the means for nicely modifying comments.
+
+;;; The function rebox-comment discovers the extent of the boxed comments
+;;; near the cursor, possibly refills the text, then adjusts the comment
+;;; box style.  The function rebox-region does the same, except that it
+;;; takes the current region as a boxed comment.  Numeric prefixes are
+;;; used to add or remove a box, change its style (language, quality or
+;;; type), or to prevent refilling of its text.  A minus sign alone as
+;;; prefix asks for interactive style selection.
+
+;;; For most Emacs language editing modes, refilling does not make sense
+;;; outside comments, so you may redefine the M-q command and link it to
+;;; this file.  For example, I use this in my .emacs file:
+
 ;;;	(setq c-mode-hook
 ;;;	      '(lambda ()
 ;;;		 (define-key c-mode-map "\M-q" 'rebox-comment)))
 ;;;	(autoload 'rebox-comment "rebox" nil t)
 ;;;	(autoload 'rebox-region "rebox" nil t)
-;;;
-;;; The cursor should be within a comment before any of these
-;;; commands, or else it should be between two comments, in which case
-;;; the command applies to the next comment.  When the command is
-;;; given without prefix, the current comment box style is recognized
-;;; from the comment itself as far as possible, and preserved.  A
-;;; prefix may be used to force a particular box style.  A style is
-;;; made up of three attributes: a language (the hundreds digit), a
-;;; quality (the tens digit) and a type (the units digit).  A zero or
-;;; negative flag value changes the default box style to its absolute
-;;; value.  Zero digits in default style, when not overriden in flag,
-;;; asks for recognition of corresponding attributes from the current
-;;; box.  C-u avoids refilling the text, using the default box style.
-;;;
-;;; Box language is associated with comment delimiters.  Values are
-;;; 100 for none or unknown, 200 for `/*' and `*/' as in plain C, 300
-;;; for '//' as in C++, 400 for `#' as in most scripting languages,
-;;; 500 for `;' as in LISP or assembler and 600 for `%' as in TeX or
-;;; PostScript.
-;;;
-;;; Box quality differs according to language.  For unknown languages
-;;; (100) or for the C language (200), values are 10 for simple, 20 or
-;;; 30 for rounded, and 40 for starred.  For all others, box quality
-;;; indicates the thickness in characters of the left and right sides
-;;; of the box: values are 10, 20, 30 or 40 for 1, 2, 3 or 4
-;;; characters wide.  C++ quality 10 is always promoted to 20.
-;;;
-;;; Box type values are 1 for fully opened boxes for which boxing is
-;;; done only for the left and right but not for top or bottom, 2 for
-;;; half single lined boxes for which boxing is done on all sides
-;;; except top, 3 for fully single lined boxes for which boxing is
-;;; done on all sides, 4 for half double lined boxes which is like
-;;; type 2 but more bold, or 5 for fully double lined boxes which is
-;;; like type 3 but more bold.
-;;;
-;;; Roughly said, simple quality boxes (10) use comment delimiters to
-;;; left and right of each comment line, and also for the top or
-;;; bottom line when applicable.  Rounded quality boxes (20 or 30) try
-;;; to suggest rounded corners in boxes.  Starred quality boxes (40)
-;;; mostly use a left margin of asterisks or X'es, and use them also
-;;; in box surroundings.  Experiment a little to see what happens.
-;;;
+
+;;; The cursor should be within a comment before any of these commands,
+;;; or else it should be between two comments, in which case the command
+;;; applies to the next comment.  When the command is given without prefix,
+;;; the current comment box style is recognized from the comment itself
+;;; as far as possible, and preserved.	A prefix may be used to force
+;;; a particular box style.  A style is made up of three attributes: a
+;;; language (the hundreds digit), a quality (the tens digit) and a type
+;;; (the units digit).	A zero or negative flag value changes the default
+;;; box style to its absolute value.  Zero digits in default style,
+;;; when not overriden in flag, asks for recognition of corresponding
+;;; attributes from the current box.  `C-u' avoids refilling the text,
+;;; using the default box style.  `C-u -' defines the style interactively.
+
+;;; Box language is associated with comment delimiters.  Values are 100
+;;; for none or unknown, 200 for `/*' and `*/' as in plain C, 300 for
+;;; '//' as in C++, 400 for `#' as in most scripting languages, 500 for
+;;; `;' as in LISP or assembler and 600 for `%' as in TeX or PostScript.
+
+;;; Box quality differs according to language.	For unknown languages (100)
+;;; or for the C language (200), values are 10 for simple, 20 or 30 for
+;;; rounded, and 40 for starred.  For all others, box quality indicates
+;;; the thickness in characters of the left and right sides of the box:
+;;; values are 10, 20, 30 or 40 for 1, 2, 3 or 4 characters wide.  C++
+;;; quality 10 is always promoted to 20.  Roughly said, simple quality
+;;; boxes (10) use comment delimiters to left and right of each comment
+;;; line, and also for the top or bottom line when applicable.	Rounded
+;;; quality boxes (20 or 30) try to suggest rounded corners in boxes.
+;;; Starred quality boxes (40) mostly use a left margin of asterisks or
+;;; X'es, and use them also in box surroundings.  Experiment a little to
+;;; see what happens.
+
+;;; Box type values are 1 for fully opened boxes for which boxing is done
+;;; only for the left and right but not for top or bottom, 2 for half
+;;; single lined boxes for which boxing is done on all sides except top,
+;;; 3 for fully single lined boxes for which boxing is done on all sides,
+;;; 4 for half double lined boxes which is like type 2 but more bold,
+;;; or 5 for fully double lined boxes which is like type 3 but more bold.
+
 ;;; The special style 221 or 231 is worth a note, because it is fairly
 ;;; common: the whole C comment stays between a single opening `/*'
 ;;; and a single closing `*/'.  The special style 111 deletes a box.
-;;; The initial default style is 023 so, unless overriden, comments
-;;; are put in single lined boxes, C comments are of rounded quality.
-;;;
-;;; I first observed rounded corners, as used in style 223 boxes, in
-;;; code from Warren Tucker <wht@n4hgf.mt-park.ga.us>.
+;;; The initial default style is 023 so, unless overriden, comments are
+;;; put in single lined boxes, C comments are of rounded quality.
 
 (defvar rebox-default-style 0 "*Preferred style for box comments.")
 
+;;; Help strings for prompting or error messages.
+
+(defconst REBOX_HELP_FOR_LANGUAGE
+  "Box language is 100-none, 200-/*, 300-//, 400-#, 500-;, 600-%%")
+(defconst REBOX_LANGUAGE_NONE 100)
+(defconst REBOX_LANGUAGE_C 200)
+(defconst REBOX_LANGUAGE_C++ 300)
+(defconst REBOX_LANGUAGE_AWK 400)
+(defconst REBOX_LANGUAGE_LISP 500)
+(defconst REBOX_LANGUAGE_TEX 600)
+
+(defun rebox-help-string-for-language (language)
+  (cond ((= language 0) "default language")
+	((= language REBOX_LANGUAGE_NONE) "no language")
+	((= language REBOX_LANGUAGE_C) "plain C")
+	((= language REBOX_LANGUAGE_C++) "C++")
+	((= language REBOX_LANGUAGE_AWK) "sh/Perl/make")
+	((= language REBOX_LANGUAGE_LISP) "LISP/assembler")
+	((= language REBOX_LANGUAGE_TEX) "TeX/PostScript")
+	(t "<Unknown Language>")))
+
+(defconst REBOX_HELP_FOR_QUALITY
+  "Box quality/width is 10-simple, 20-rounded, 30-rounded or 40-starred")
+(defconst REBOX_QUALITY_SIMPLE_ONE 10)
+(defconst REBOX_QUALITY_ROUNDED_TWO 20)
+(defconst REBOX_QUALITY_ROUNDED_THREE 30)
+(defconst REBOX_QUALITY_STARRED_FOUR 40)
+
+(defun rebox-help-string-for-quality (quality)
+  (cond ((= quality 0) "default quality")
+	((= quality REBOX_QUALITY_SIMPLE_ONE) "square or 1-wide")
+	((= quality REBOX_QUALITY_ROUNDED_TWO) "rounded or 2-wide")
+	((= quality REBOX_QUALITY_ROUNDED_THREE) "rounded or 3-wide")
+	((= quality REBOX_QUALITY_STARRED_FOUR) "starred or 4-wide")
+	(t "<Unknown Quality>")))
+
+(defconst REBOX_HELP_FOR_TYPE
+  "Box type is 1-open, 2-half-single, 3-single, 4-half-double or 5-double")
+(defconst REBOX_TYPE_OPEN 1)
+(defconst REBOX_TYPE_HALF_SINGLE 2)
+(defconst REBOX_TYPE_SINGLE 3)
+(defconst REBOX_TYPE_HALF_DOUBLE 4)
+(defconst REBOX_TYPE_DOUBLE 5)
+
+(defun rebox-help-string-for-type (type)
+  (cond ((= type 0) "default type")
+	((= type REBOX_TYPE_OPEN) "opened box")
+	((= type REBOX_TYPE_HALF_SINGLE) "half normal")
+	((= type REBOX_TYPE_SINGLE) "full normal")
+	((= type REBOX_TYPE_HALF_DOUBLE) "half bold")
+	((= type REBOX_TYPE_DOUBLE) "full bold")
+	(t "<Unknown Type>")))
+
+(defconst REBOX_MAX_LANGUAGE 6)
+(defconst REBOX_MAX_QUALITY 4)
+(defconst REBOX_MAX_TYPE 5)
+
+;;; Request the style interactively, using the minibuffer.
+
+(defun rebox-ask-for-style ()
+  (let (key language quality type)
+    (while (not language)
+      (message REBOX_HELP_FOR_LANGUAGE)
+      (setq key (read-char))
+      (if (and (>= key ?0) (<= key (+ ?0 REBOX_MAX_LANGUAGE)))
+	  (setq language (- key ?0))))
+    (while (not quality)
+      (message REBOX_HELP_FOR_QUALITY)
+      (setq key (read-char))
+      (if (and (>= key ?0) (<= key (+ ?0 REBOX_MAX_QUALITY)))
+	  (setq quality (- key ?0))))
+    (while (not type)
+      (message REBOX_HELP_FOR_TYPE)
+      (setq key (read-char))
+      (if (and (>= key ?0) (<= key (+ ?0 REBOX_MAX_TYPE)))
+	  (setq type (- key ?0))))
+    (+ (* 100 language) (* 10 quality) type)))
+
 ;;; Write some TEXT followed by an edited STYLE value into the minibuffer.
 
 (defun rebox-show-style (text style)
-  (let (language quality type)
-    (setq text (concat text))
-    (setq language (* (/ style 100) 100))
-    (setq quality (* (% (/ style 10) 10) 10))
-    (setq type (% style 10))
-    (message (concat text (format " (%03d)" style)
-		     ": " (cond ((= language 000) "default language")
-				((= language 100) "no language")
-				((= language 200) "plain C")
-				((= language 300) "C++")
-				((= language 400) "sh/Perl/make")
-				((= language 500) "LISP/assembler")
-				((= language 600) "TeX/PostScript")
-				(t "<UNKNOWN LANGUAGE>"))
-		     ", " (cond ((= quality 00) "default quality")
-				((= quality 10) "square or 1-wide")
-				((= quality 20) "rounded or 2-wide")
-				((= quality 30) "rounded or 3-wide")
-				((= quality 40) "starred or 4-wide")
-				(t "<UNKNOWN QUALITY>"))
-		     ", " (cond ((= type 0) "default type")
-				((= type 1) "opened box")
-				((= type 2) "half normal")
-				((= type 3) "full normal")
-				((= type 4) "half bold")
-				((= type 5) "full bold")
-				(t "<UNKNOWN TYPE>"))))))
+  (message
+   (concat text (format " (%03d)" style)
+	   ": " (rebox-help-string-for-language (* (/ style 100) 100))
+	   ", " (rebox-help-string-for-quality (* (% (/ style 10) 10) 10))
+	   ", " (rebox-help-string-for-type (% style 10)))))
 
 ;;; Validate FLAG and usually return t if not interrupted by errors.
-;;; But if FLAG is zero or negative, change default box style, then
+;;; But if FLAG is zero or negative, then change default box style and
 ;;; return nil.
 
 (defun rebox-validate-flag (flag)
@@ -118,16 +171,12 @@
 
   (if (numberp flag)
       (let ((value (if (< flag 0) (- flag) flag)))
-	(if (> (/ value 100) 6)
-	    (error "\
-Box language should be 100-none, 200-/*, 300-//, 400-#, 500-;, 600-%%"))
-	(if (> (% (/ value 10) 10) 4)
-	    (error "\
-Box quality should be 10-simple, 20-rounded, 30-rounded or 40-starred"))
-	(if (> (% value 10) 5)
-	    (error "\
-Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
-		   ))))
+	(if (> (/ value 100) REBOX_MAX_LANGUAGE)
+	    (error REBOX_HELP_FOR_LANGUAGE))
+	(if (> (% (/ value 10) 10) REBOX_MAX_QUALITY)
+	    (error REBOX_HELP_FOR_QUALITY))
+	(if (> (% value 10) REBOX_MAX_TYPE)
+	    (error REBOX_HELP_FOR_TYPE))))
 
   ;; Change default box style if requested.
 
@@ -185,34 +234,34 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
     margin))
 
 ;;; Return a regexp to match the start or end of a comment for some
-;;; LANGUAGE.
+;;; LANGUAGE, leaving the comment marks themselves available in \1.
 
 ;; FIXME: Recognize style 1** boxes.
 
 (defun rebox-regexp-start (language)
-  (cdr (assoc language '((0 . "^[ \t]*\\(/\\*\\|//+\\|#+\\|;+\\|%+\\)")
-			 (100 . "^")
-			 (200 . "^[ \t]*\\(/\\*\\)")
-			 (300 . "^[ \t]*\\(//+\\)")
-			 (400 . "^[ \t]*\\(#+\\)")
-			 (500 . "^[ \t]*\\(;+\\)")
-			 (600 . "^[ \t]*\\(%+\\)")))))
+  (cond ((= language 0) "^[ \t]*\\(/\\*\\|//+\\|#+\\|;+\\|%+\\)")
+	((= language REBOX_LANGUAGE_NONE) "^\\(\\)")
+	((= language REBOX_LANGUAGE_C) "^[ \t]*\\(/\\*\\)")
+	((= language REBOX_LANGUAGE_C++) "^[ \t]*\\(//+\\)")
+	((= language REBOX_LANGUAGE_AWK) "^[ \t]*\\(#+\\)")
+	((= language REBOX_LANGUAGE_LISP) "^[ \t]*\\(;+\\)")
+	((= language REBOX_LANGUAGE_TEX) "^[ \t]*\\(%+\\)")))
 
 (defun rebox-regexp-end (language)
-  (cdr (assoc language '((0 . "\\(\\*/\\|//+\\|#+\\|;+\\|%+\\)[ \t]*$")
-			 (100 . "$")
-			 (200 . "\\(\\*/\\)[ \t]*$")
-			 (300 . "\\(//+\\)[ \t]*$")
-			 (400 . "\\(#+\\)[ \t]*$")
-			 (500 . "\\(;+\\)[ \t]*$")
-			 (600 . "\\(%+\\)[ \t]*$")))))
+  (cond ((= language 0) "\\(\\*/\\|//+\\|#+\\|;+\\|%+\\)[ \t]*$")
+	((= language REBOX_LANGUAGE_NONE) "\\(\\)$")
+	((= language REBOX_LANGUAGE_C) "\\(\\*/\\)[ \t]*$")
+	((= language REBOX_LANGUAGE_C++) "\\(//+\\)[ \t]*$")
+	((= language REBOX_LANGUAGE_AWK) "\\(#+\\)[ \t]*$")
+	((= language REBOX_LANGUAGE_LISP) "\\(;+\\)[ \t]*$")
+	((= language REBOX_LANGUAGE_TEX) "\\(%+\\)[ \t]*$")))
 
 ;;; By looking at the text starting at the cursor position, guess the
 ;;; language in use, and return it.
 
 (defun rebox-guess-language ()
-  (let ((language 100)
-	(value 600))
+  (let ((language REBOX_LANGUAGE_NONE)
+	(value (* 100 REBOX_MAX_LANGUAGE)))
     (while (not (zerop value))
       (if (looking-at (rebox-regexp-start value))
 	  (progn
@@ -256,23 +305,26 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 
       ;; - find the end of this comment
 
-      (if (= language 200)
+      (if (= language REBOX_LANGUAGE_C)
 	  (progn
 	    (search-forward "*/")
 	    (if (not (looking-at "[ \t]*$"))
 		(error "text after end of comment"))))
-      (beginning-of-line)
-      (forward-line 1)
+      (end-of-line)
+      (if (eobp)
+	  (insert "\n")
+	(forward-char 1))
       (setq end (point))
 
       ;; - try to extend the comment block backwards
 
       (goto-char start)
       (while (and (not (bobp))
-		  (if (= language 200)
+		  (if (= language REBOX_LANGUAGE_C)
 		      (progn
 			(skip-chars-backward " \t\n")
-			(if (> (point) 2)
+			(if (and (looking-at "[ \t]*\n[ \t]*/\\*")
+				 (> (point) 2))
 			    (progn
 			      (backward-char 2)
 			      (if (looking-at "\\*/")
@@ -291,7 +343,7 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 
       (goto-char end)
       (while (looking-at (rebox-regexp-start language))
-	(if (= language 200)
+	(if (= language REBOX_LANGUAGE_C)
 	    (progn
 	      (re-search-forward "[ \t]*/\\*")
 	      (re-search-forward "\\*/")
@@ -318,148 +370,150 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 
     ;; - decide the elements of the box being produced
 
-    (cond ((= language 100)
+    (cond ((= language REBOX_LANGUAGE_NONE)
 	   ;; - planify a comment for no language in particular
 
-	   (cond ((= quality 10)
+	   (cond ((= quality REBOX_QUALITY_SIMPLE_ONE)
 		  ;; - planify a simple box
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 (setq nw "") (setq sw "")
 			 (setq ww "") (setq ee ""))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "+-") (setq ss ?-) (setq se "-+"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw "+-") (setq nn ?-) (setq ne "-+")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "+-") (setq ss ?-) (setq se "-+"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "*=") (setq ss ?=) (setq se "=*"))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "*=") (setq nn ?=) (setq ne "=*")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "*=") (setq ss ?=) (setq se "=*"))))
 
-		 ((or (= quality 20) (= quality 30))
+		 ((or (= quality REBOX_QUALITY_ROUNDED_TWO)
+		      (= quality REBOX_QUALITY_ROUNDED_THREE))
 		  ;; - planify a rounded box
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 (setq nw "") (setq sw "")
 			 (setq ww "| ") (setq ee " |"))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "`-") (setq ss ?-) (setq se "-'"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw ".-") (setq nn ?-) (setq ne "-.")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "`-") (setq ss ?-) (setq se "-'"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "")
 			 (setq ww "| " )              (setq ee " |" )
 			 (setq sw "\\=") (setq ss ?=) (setq se "=/" ))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "/=" ) (setq nn ?=) (setq ne "=\\")
 			 (setq ww "| " )              (setq ee " |" )
 			 (setq sw "\\=") (setq ss ?=) (setq se "=/" ))))
 
-		 ((= quality 40)
+		 ((= quality REBOX_QUALITY_STARRED_FOUR)
 		  ;; - planify a starred box
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 (setq nw "") (setq sw "")
 			 (setq ww "| ") (setq ee ""))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "")
 			 (setq ww "* ")              (setq ee " *")
 			 (setq sw "**") (setq ss ?*) (setq se "**"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw "**") (setq nn ?*) (setq ne "**")
 			 (setq ww "* ")              (setq ee " *")
 			 (setq sw "**") (setq ss ?*) (setq se "**"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "")
 			 (setq ww "X ")              (setq ee " X")
 			 (setq sw "XX") (setq ss ?X) (setq se "XX"))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "XX") (setq nn ?X) (setq ne "XX")
 			 (setq ww "X ")              (setq ee " X")
 			 (setq sw "XX") (setq ss ?X) (setq se "XX"))))))
 
-	  ((= language 200)
+	  ((= language REBOX_LANGUAGE_C)
 	   ;; - planify a comment for C
 
-	   (cond ((= quality 10)
+	   (cond ((= quality REBOX_QUALITY_SIMPLE_ONE)
 		  ;; - planify a simple C comment
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 (setq nw "") (setq sw "")
 			 (setq ww "/* ") (setq ee " */"))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "")
 			 (setq ww "/* ")              (setq ee " */")
 			 (setq sw "/* ") (setq ss ?-) (setq se " */"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw "/* ") (setq nn ?-) (setq ne " */")
 			 (setq ww "/* ")              (setq ee " */")
 			 (setq sw "/* ") (setq ss ?-) (setq se " */"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "")
 			 (setq ww "/* ")              (setq ee " */")
 			 (setq sw "/* ") (setq ss ?=) (setq se " */"))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "/* ") (setq nn ?=) (setq ne " */")
 			 (setq ww "/* ")              (setq ee " */")
 			 (setq sw "/* ") (setq ss ?=) (setq se " */"))))
 
-		 ((or (= quality 20) (= quality 30))
+		 ((or (= quality REBOX_QUALITY_ROUNDED_TWO)
+		      (= quality REBOX_QUALITY_ROUNDED_THREE))
 		  ;; - planify a rounded C comment
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 ;; ``open rounded'' is a special case
 			 (setq nw "") (setq sw "")
 			 (setq ww "   ") (setq ee ""))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "/*") (setq nn ? ) (setq ne " .")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "`-") (setq ss ?-) (setq se "*/"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw "/*") (setq nn ?-) (setq ne "-.")
 			 (setq ww "| ")              (setq ee " |")
 			 (setq sw "`-") (setq ss ?-) (setq se "*/"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "/*" ) (setq nn ? ) (setq ne " \\")
 			 (setq ww "| " )              (setq ee " |" )
 			 (setq sw "\\=") (setq ss ?=) (setq se "*/" ))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "/*" ) (setq nn ?=) (setq ne "=\\")
 			 (setq ww "| " )              (setq ee " |" )
 			 (setq sw "\\=") (setq ss ?=) (setq se "*/" ))))
 
-		 ((= quality 40)
+		 ((= quality REBOX_QUALITY_STARRED_FOUR)
 		  ;; - planify a starred C comment
 
-		  (cond ((= type 1)
+		  (cond ((= type REBOX_TYPE_OPEN)
 			 (setq nw "/* ") (setq nn ? ) (setq ne "")
 			 (setq ww " * ")              (setq ee "")
 			 (setq sw " */") (setq ss ? ) (setq se ""))
-			((= type 2)
+			((= type REBOX_TYPE_HALF_SINGLE)
 			 (setq nw "/* ") (setq nn ? ) (setq ne " *")
 			 (setq ww " * ")              (setq ee " *")
 			 (setq sw " **") (setq ss ?*) (setq se "**/"))
-			((= type 3)
+			((= type REBOX_TYPE_SINGLE)
 			 (setq nw "/**") (setq nn ?*) (setq ne "**")
 			 (setq ww " * ")              (setq ee " *")
 			 (setq sw " **") (setq ss ?*) (setq se "**/"))
-			((= type 4)
+			((= type REBOX_TYPE_HALF_DOUBLE)
 			 (setq nw "/* " ) (setq nn ? ) (setq ne " *\\")
 			 (setq ww "|* " )              (setq ee " *|" )
 			 (setq sw "\\**") (setq ss ?*) (setq se "**/" ))
-			((= type 5)
+			((= type REBOX_TYPE_DOUBLE)
 			 (setq nw "/**" ) (setq nn ?*) (setq ne "**\\")
 			 (setq ww "|* " )              (setq ee " *|" )
 			 (setq sw "\\**") (setq ss ?*) (setq se "**/" ))))))
@@ -467,30 +521,31 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 	  (t
 	   ;; - planify a comment for all other things
 
-	   (if (and (= language 300) (= quality 10))
-	       (setq quality 20))
-	   (setq x (cond ((= language 300) ?/)
-			 ((= language 400) ?#)
-			 ((= language 500) ?\;)
-			 ((= language 600) ?%)))
+	   (if (and (= language REBOX_LANGUAGE_C++)
+		    (= quality REBOX_QUALITY_SIMPLE_ONE))
+	       (setq quality REBOX_QUALITY_ROUNDED_TWO))
+	   (setq x (cond ((= language REBOX_LANGUAGE_C++) ?/)
+			 ((= language REBOX_LANGUAGE_AWK) ?#)
+			 ((= language REBOX_LANGUAGE_LISP) ?\;)
+			 ((= language REBOX_LANGUAGE_TEX) ?%)))
 	   (setq xx (make-string (/ quality 10) x))
 	   (setq ww (concat xx " "))
-	   (cond ((= type 1)
+	   (cond ((= type REBOX_TYPE_OPEN)
 		  (setq nw "") (setq sw "") (setq ee ""))
-		 ((= type 2)
+		 ((= type REBOX_TYPE_HALF_SINGLE)
 		  (setq ee (concat " " xx))
 		  (setq nw "")
 		  (setq sw ww) (setq ss ?-) (setq se ee))
-		 ((= type 3)
+		 ((= type REBOX_TYPE_SINGLE)
 		  (setq ee (concat " " xx))
 		  (setq nw ww) (setq nn ?-) (setq ne ee)
 		  (setq sw ww) (setq ss ?-) (setq se ee))
-		 ((= type 4)
+		 ((= type REBOX_TYPE_HALF_DOUBLE)
 		  (setq ee (concat " " xx))
 		  (setq xx (make-string (1+ (/ quality 10)) x))
 		  (setq nw "")
 		  (setq sw xx) (setq ss x) (setq se xx))
-		 ((= type 5)
+		 ((= type REBOX_TYPE_DOUBLE)
 		  (setq ee (concat " " xx))
 		  (setq xx (make-string (1+ (/ quality 10)) x))
 		  (setq nw xx) (setq nn x) (setq ne xx)
@@ -508,7 +563,10 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
     ;; - construct the box comment, from top to bottom
 
     (goto-char (point-min))
-    (if (and (= language 200) (or (= quality 20) (= quality 30)) (= type 1))
+    (if (and (= language REBOX_LANGUAGE_C)
+	     (or (= quality REBOX_QUALITY_ROUNDED_TWO)
+		 (= quality REBOX_QUALITY_ROUNDED_THREE))
+	     (= type REBOX_TYPE_OPEN))
 	(progn
 	  ;; - construct an 33 style comment
 
@@ -567,74 +625,76 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
   (let ((undo-list buffer-undo-list)
 	(marked-point (point-marker))
 	(language (progn (goto-char (point-min)) (rebox-guess-language)))
-	(quality (* (% (/ rebox-default-style 10) 10) 10))
-	(type 1))
+	(quality 0)
+	(type 0))
 
     (untabify (point-min) (point-max))
 
-    ;; Remove all the comment marks, and move all the text rigidly
-    ;; to the left to insure the left margin stays at the same
-    ;; place.  At the same time, recognize and save the box style in
-    ;; TYPE and box quality in QUALITY.
+    ;; Remove all the comment marks, and move all the text rigidly to the
+    ;; left for insuring that the left margin stays at the same place.
+    ;; At the same time, try recognizing the box style, saving its quality
+    ;; in QUALITY and its type in TYPE.  (LANGUAGE is already guessed.)
 
-    (let ((previous-margin (rebox-left-margin))
+    (let ((indent-tabs-mode nil)
+	  (previous-margin (rebox-left-margin))
 	  actual-margin)
 
       ;; FIXME: Cleanup style 1** boxes.
-
-      ;; FIXME: Cleanup style 241 boxes.
-
-      ;; FIXME: quality should be deduced instead
-      (if (zerop quality)
-	  (setq quality 20))
+      ;; FIXME: Recognize really all cases of type and quality.
 
       ;; - remove all comment marks
 
-      (if (= language 100)
+      (if (= language REBOX_LANGUAGE_NONE)
 	  nil
 	(goto-char (point-min))
 	(while (re-search-forward (rebox-regexp-start language) nil t)
-	  (replace-match (make-string (- (match-end 1) (match-beginning 1)) ? )
-			 t t))
+	  (goto-char (match-beginning 1))
+	  (delete-region (point) (match-end 1))
+	  (insert (make-string (- (match-end 1) (point)) ? )))
 	(goto-char (point-min))
 	(while (re-search-forward (rebox-regexp-end language) nil t)
 	  (replace-match "" t t)))
 
-      (if (= language 200)
+      (if (= language REBOX_LANGUAGE_C)
 	  (progn
 	    (goto-char (point-min))
 	    (while (re-search-forward "\\*/ */\\*" nil t)
 	      (replace-match "  " t t))
 
 	    (goto-char (point-min))
-	    (while (re-search-forward "^\\( *\\)|\\*\\(.*\\)\\*| *$"
-				      nil t)
-	      (setq quality 40)
-	      (setq type 5)
+	    (while (re-search-forward "^\\( *\\)|\\*\\(.*\\)\\*| *$" nil t)
+	      (setq quality REBOX_QUALITY_STARRED_FOUR)
+	      (setq type REBOX_TYPE_DOUBLE)
 	      (replace-match "\\1  \\2" t))
 
 	    (goto-char (point-min))
-	    (while (re-search-forward "^\\( *\\)\\*\\(.*\\)\\* *$"
-				      nil t)
-	      (setq quality 40)
-	      (setq type 3)
+	    (while (re-search-forward "^\\( *\\)\\*\\(.*\\)\\* *$" nil t)
+	      (setq quality REBOX_QUALITY_STARRED_FOUR)
+	      (setq type REBOX_TYPE_SINGLE)
 	      (replace-match "\\1 \\2" t))
 
 	    (goto-char (point-min))
 	    (while (re-search-forward "^\\( *\\)|\\(.*\\)| *$" nil t)
-	      (setq quality 20)
-	      (replace-match "\\1 \\2" t))))
+	      (setq quality REBOX_QUALITY_ROUNDED_TWO)
+	      (replace-match "\\1 \\2" t))
+
+	    (goto-char (point-min))
+	    (if (zerop quality)
+		(while (re-search-forward "^\\( +\\)\\* " nil t)
+		  (setq quality REBOX_QUALITY_STARRED_FOUR)
+		  (setq type REBOX_TYPE_OPEN)
+		  (replace-match "\\1  " t)))))
 
       ;; - remove the first dashed or starred line
 
       (goto-char (point-min))
       (if (looking-at "^ *\\(--+\\|\\*\\*+\\)[.\+\\]? *\n")
 	  (progn
-	    (setq type 3)
+	    (setq type REBOX_TYPE_SINGLE)
 	    (replace-match "" t t))
 	(if (looking-at "^ *\\(==\\|XX+\\|##+\\|;;+\\)[.\+\\]? *\n")
 	    (progn
-	      (setq type 5)
+	      (setq type REBOX_TYPE_DOUBLE)
 	      (replace-match "" t t))))
 
       ;; - remove the last dashed or starred line
@@ -643,23 +703,23 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
       (previous-line 1)
       (if (looking-at "^ *[`\+\\]?*--+ *\n")
 	  (progn
-	    (if (= type 1)
-		(setq type 2))
+	    (if (= type REBOX_TYPE_OPEN)
+		(setq type REBOX_TYPE_HALF_SINGLE))
 	    (replace-match "" t t))
 	(if (looking-at "^ *[`\+\\]?*\\(==+\\|##+\\|;;+\\) *\n")
 	    (progn
-	      (if (= type 1)
-		  (setq type 4))
+	      (if (= type REBOX_TYPE_OPEN)
+		  (setq type REBOX_TYPE_HALF_DOUBLE))
 	      (replace-match "" t t))
 	  (if (looking-at "^ *\\*\\*+[.\+\\]? *\n")
 	      (progn
-		(setq quality 40)
-		(setq type 2)
+		(setq quality REBOX_QUALITY_STARRED_FOUR)
+		(setq type REBOX_TYPE_HALF_SINGLE)
 		(replace-match "" t t))
 	    (if (looking-at "^ *XX+[.\+\\]? *\n")
 		(progn
-		  (setq quality 40)
-		  (setq type 4)
+		  (setq quality REBOX_QUALITY_STARRED_FOUR)
+		  (setq type REBOX_TYPE_HALF_DOUBLE)
 		  (replace-match "" t t))))))
 
       ;; - remove all spurious whitespace
@@ -688,28 +748,34 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 	  (indent-rigidly (point-min) (point-max)
 			  (- previous-margin actual-margin))))
 
-    ;; Override box style according to FLAG.
+    ;; Override box style according to FLAG or chosen default style.
+    ;; Else, use either recognized style elements or built-in defaults.
 
-    (if (and (numberp flag) (not (zerop (/ flag 100))))
-	(setq language (* (/ flag 100) 100))
-      (if (not (zerop (/ rebox-default-style 100)))
-	  (setq language (* (/ rebox-default-style 100) 100))))
+    (cond ((and (numberp flag) (not (zerop (/ flag 100))))
+	   (setq language (* (/ flag 100) 100)))
+	  ((not (zerop (/ rebox-default-style 100)))
+	   (setq language (* (/ rebox-default-style 100) 100))))
 
-    (if (and (numberp flag) (not (zerop (% (/ flag 10) 10))))
-	(setq quality (* (% (/ flag 10) 10) 10))
-      (if (not (zerop (% (/ rebox-default-style 10) 10)))
-	  (setq quality (* (% (/ rebox-default-style 10) 10) 10))))
+    (cond ((and (numberp flag) (not (zerop (% (/ flag 10) 10))))
+	   (setq quality (* (% (/ flag 10) 10) 10)))
+	  ((not (zerop (% (/ rebox-default-style 10) 10)))
+	   (setq quality (* (% (/ rebox-default-style 10) 10) 10)))
+	  ((zerop quality)
+	   (setq quality REBOX_QUALITY_ROUNDED_TWO)))
 
-    (if (and (numberp flag) (not (zerop (% flag 10))))
-	(setq type (% flag 10))
-      (if (not (zerop (% rebox-default-style 10)))
-	  (setq type (% rebox-default-style 10))))
+    (cond ((and (numberp flag) (not (zerop (% flag 10))))
+	   (setq type (% flag 10)))
+	  ((not (zerop (% rebox-default-style 10)))
+	   (setq type (% rebox-default-style 10)))
+	  ((zerop type)
+	   (setq type 1)))
 
     ;; Possibly refill, then reconstruct the comment box.
 
-    (rebox-reconstruct (not (and flag (listp flag)))
-		       (rebox-left-margin)
-		       language quality type)
+    (let ((indent-tabs-mode nil))
+      (rebox-reconstruct (not (and flag (listp flag)))
+			 (rebox-left-margin)
+			 language quality type))
 
     ;; Retabify to the left only (adapted from tabify.el).
 
@@ -717,8 +783,7 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 	(progn
 	  (goto-char (point-min))
 	  (while (re-search-forward "^[ \t][ \t]+" nil t)
-	    (let ((column (current-column))
-		  (indent-tabs-mode t))
+	    (let ((column (current-column)))
 	      (delete-region (match-beginning 0) (point))
 	      (indent-to column)))))
 
@@ -752,7 +817,7 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 	(setq c-argdecl-indent 5)
 	(setq c-label-offset -2)
 	(setq c-tab-always-indent t)
-	(setq rebox-default-style 020)
+	(setq rebox-default-style REBOX_QUALITY_ROUNDED_TWO)
 	(message "C mode: GNU style"))
 
     (setq c-mode-taarna-style t)
@@ -762,13 +827,15 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
     (setq c-argdecl-indent 4)
     (setq c-label-offset -4)
     (setq c-tab-always-indent t)
-    (setq rebox-default-style 012)
+    (setq rebox-default-style
+	  (+ REBOX_QUALITY_SIMPLE_ONE REBOX_TYPE_HALF_SINGLE))
     (message "C mode: Taarna style")))
 
 ;;; Rebox the current region.
 
 (defun rebox-region (flag)
   (interactive "P")
+  (if (eq flag '-) (setq flag (rebox-ask-for-style)))
   (if (rebox-validate-flag flag)
       (save-restriction
 	(narrow-to-region (region-beginning) (region-end))
@@ -778,6 +845,7 @@ Box type should be 1-open, 2-half-single, 3-single, 4-half-double or 5-double"
 
 (defun rebox-comment (flag)
   (interactive "P")
+  (if (eq flag '-) (setq flag (rebox-ask-for-style)))
   (if (rebox-validate-flag flag)
       (save-restriction
 	(rebox-find-and-narrow)
